MODULE CalcZX; (* ZX Spectrum's ROM calculator: compact real arithmetic. *)
IMPORT SYSTEM, Asm;

PROCEDURE -AANoRETCHK0 "#undef __RETCHK";
PROCEDURE -AANoRETCHK1 "#define __RETCHK";

(* http://zxpress.ru/article.php?id=6609

#2D22 - STK-DIGIT.  =  ASCII ->  
( );
#2D28 -  STACK-A.    ->    (    "
");
#2D2B - STACK-BC. BC ->  (-''-);
#2AB1 - STK-STORE. A,E,D,C,B ->    5-  
;      (  A=0);
#2ABB -          (  ,
    ).
*)

PROCEDURE -jump_true*  "__asm .db 0x00 __endasm"; (** jump true *)
(*
#01 - exchange
#02 - delete
*)
PROCEDURE -subtract*  "__asm .db 0x03 __endasm"; (** subtract (; X-Y) *)
PROCEDURE -mult*      "__asm .db 0x04 __endasm"; (** mult (; X*Y) *)
PROCEDURE -div*       "__asm .db 0x05 __endasm"; (** div (; Y/X) *)
PROCEDURE -to_power*  "__asm .db 0x06 __endasm"; (** to-power (  ; X^Y) *)
PROCEDURE -or*        "__asm .db 0x03 __endasm"; (** or *)
(*
#08 - no-&-no
#09 - no-l-eq
#0A - no-gr-eq
#0B - nos-eql
#0C - no-gtr
#0D - no-less
#0E - nos-eql
*)
PROCEDURE -add* "__asm .db 0x0F __endasm"; (** add (; X+Y) *)
(*
#10 - str-&-no
#11 - str-l-eq
#12 - str-gr-eq
#13 - strs-negl
#14 - str-gtr
#15 - str-less
#16 - str-gtr
#17 - strs-add
#18 - val$ (;   fp-calc-2)
#19 - usr-s
#1A - read-in
#1B - neg
#1C - code
#1D - val (;   fp-calc-2)
#1E - len
*)
PROCEDURE -sin*   "__asm .db 0x1F __endasm"; (** #1F - sin *)
PROCEDURE -cos*   "__asm .db 0x20 __endasm"; (** #20 - cos *)
PROCEDURE -tan*   "__asm .db 0x21 __endasm"; (** #21 - tan *)
PROCEDURE -asn*   "__asm .db 0x22 __endasm"; (** #22 - asn *)
PROCEDURE -acs*   "__asm .db 0x23 __endasm"; (** #23 - acs *)
PROCEDURE -atn*   "__asm .db 0x24 __endasm"; (** #24 - atn *)
PROCEDURE -ln*    "__asm .db 0x25 __endasm"; (** #25 - ln *)
PROCEDURE -exp*   "__asm .db 0x26 __endasm"; (** #26 - exp *)
PROCEDURE -int*   "__asm .db 0x27 __endasm"; (** #27 - int *)
PROCEDURE -sqr*   "__asm .db 0x28 __endasm"; (** #28 - sqr *)
PROCEDURE -sgn*   "__asm .db 0x29 __endasm"; (** #29 - sgn *)
PROCEDURE -abs*   "__asm .db 0x2A __endasm"; (** #2A - abs *)
PROCEDURE -peek*  "__asm .db 0x2B __endasm"; (** #2B - peek *)
PROCEDURE -round* "__asm .db 0x2C __endasm"; (** #2C - int(Y+0.5) *)
(*
#2D - user-no
#2E - str$
#2F - chr$
*)
PROCEDURE -not*       "__asm .db 0x30 __endasm"; (** not *)
PROCEDURE -duplicate* "__asm .db 0x31 __endasm"; (** duplicate *)
(*
#33 - jump
#34 - stk-data
#35 - djnz
#36 - less-0
#37 - greater-0

#39 - get-argt
*)
PROCEDURE -trunc* "__asm .db 0x2C __endasm"; (** #3A - trunc *)
(*
#3B - fp-calc-2 (   - )
#3C - e-to-fp
#3D - restack (Y  -> Y   )
#86   - series-06
*)
PROCEDURE -_0*       "__asm .db 0xA0 __endasm"; (** 0 (   ) *)
PROCEDURE -_1*       "__asm .db 0xA1 __endasm";
PROCEDURE -_1_div_2* "__asm .db 0xA2 __endasm"; (** 1/2 *)
PROCEDURE -pi_div_2* "__asm .db 0xA3 __endasm"; (** PI/2 *)
(*
#C0   - st-mem-0
#E0   - get-mem-0
*)

PROCEDURE -Begin* "__asm rst 0x28 __endasm"; (**   *)

PROCEDURE _PushBYTE* (b: BYTE);
BEGIN
  Asm.Code("POP  HL    ");
  Asm.Code("POP  BC    ");
  Asm.Code("PUSH BC    ");
  Asm.Code("PUSH HL    ");
  Asm.Code("LD   A, C  ");
  Asm.Code("JP   0x2D28");
END _PushBYTE;

PROCEDURE -PushByte* (b: SHORTINT) "CalcZX__PushBYTE((unsigned char)(b))";

PROCEDURE _PushWORD* (w: SHORTINT);
BEGIN
  Asm.Code("POP  HL    ");
  Asm.Code("POP  BC    ");
  Asm.Code("PUSH BC    ");
  Asm.Code("PUSH HL    ");
  Asm.Code("JP   0x2D2B");
END _PushWORD;

PROCEDURE -PushWord* (w: INTEGER) "CalcZX__PushWORD((unsigned int)(w))";

PROCEDURE -PopByteAsSInt* (): BYTE "__asm jp 0x2DA2 __endasm";

PROCEDURE PopWordAsInt* (): SHORTINT;
BEGIN
  Asm.Code("CALL 0x2DA2");
  Asm.Code("LD   L, C  ");
  Asm.Code("LD   H, B  ");
END PopWordAsInt;

(*
 - FP-TO-BC.        
   BC,     (int(N+0.5));
# - FP-TO-A -  ,    1- 
 
*)

PROCEDURE -Print_PopReal* () "__asm call 0x2DE3 __endasm;"; (** PRINT-FP *)

PROCEDURE -End*   "__asm .db 0x38 __endasm;"; (** #38 - end-calc (   )*)

(*
#2BF1 -  STK-FETCH.            
    B,C,D,E,A.     
  , : DE -  , BC - ;
#2DA2 - FP-TO-BC.        
   BC,     (int(N+0.5));
#2DD5 - FP-TO-A -  ,    1- 
 ;
#2DE3 - PRINT-FP  -          (  
,    )    
   ,     
 (  ).      
      ;      
 ,             (
 );
#203C -  ,  : DE= ,
BC=.            -
    .

           
:

#16B0 -   ,    
  ;
#16BF -       ;
#16C5 -   .
*)
PROCEDURE- _init* "/*noinit*/";
END CalcZX.
